@@ -184,7 +184,7 @@ import org.apache.commons.pool.impl.GenericKeyedObjectPool.ObjectTimestampPair;
  * @author Rodney Waldhoff
  * @author Dirk Verbeeck
  * @author Sandy McArthur
- * @version $Revision: 784441 $ $Date: 2009-06-13 13:42:28 -0400 (Sat, 13 Jun 2009) $
+ * @version $Revision: 831698 $ $Date: 2009-11-01 11:33:31 -0500 (Sun, 01 Nov 2009) $
  * @since Pool 1.0
  */
 public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
@@ -861,7 +861,9 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * <p>
      * When a negative value is supplied, <tt>ceil({@link #getNumIdle})/abs({@link #getNumTestsPerEvictionRun})</tt>
      * tests will be run.  That is, when the value is <i>-n</i>, roughly one <i>n</i>th of the
-     * idle objects will be tested per run.
+     * idle objects will be tested per run. When the value is positive, the number of tests
+     * actually performed in each run will be the minimum of this value and the number of instances
+     * idle in the pool.
      *
      * @param numTestsPerEvictionRun max number of objects to examine during each evictor run.
      * @see #getNumTestsPerEvictionRun
@@ -1074,9 +1076,9 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
                         case WHEN_EXHAUSTED_GROW:
                             // allow new object to be created
                             synchronized (this) {
-                                // Make sure allocate hasn't already assigned an object
-                                // in a different thread
-                                if (latch.getPair() == null) {
+                                // Make sure another thread didn't allocate us an object
+                                // or permit a new object to be created
+                                if (latch.getPair() == null && !latch.mayCreate()) {
                                     _allocationQueue.remove(latch);
                                     _numInternalProcessing++;
                                 }
@@ -1085,8 +1087,8 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
                         case WHEN_EXHAUSTED_FAIL:
                             synchronized (this) {
                                 // Make sure allocate hasn't already assigned an object
-                                // in a different thread
-                                if (latch.getPair() != null) {
+                                // in a different thread or permitted a new object to be created
+                                if (latch.getPair() != null || latch.mayCreate()) {
                                     break;
                                 }
                                 _allocationQueue.remove(latch);
@@ -1095,17 +1097,23 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
                         case WHEN_EXHAUSTED_BLOCK:
                             try {
                                 synchronized (latch) {
-                                    if(maxWait <= 0) {
-                                        latch.wait();
-                                    } else {
-                                        // this code may be executed again after a notify then continue cycle
-                                        // so, need to calculate the amount of time to wait
-                                        final long elapsed = (System.currentTimeMillis() - starttime);
-                                        final long waitTime = maxWait - elapsed;
-                                        if (waitTime > 0)
-                                        {
-                                            latch.wait(waitTime);
+                                    // Before we wait, make sure another thread didn't allocate us an object
+                                    // or permit a new object to be created
+                                    if (latch.getPair() == null && !latch.mayCreate()) {
+                                        if(maxWait <= 0) {
+                                            latch.wait();
+                                        } else {
+                                            // this code may be executed again after a notify then continue cycle
+                                            // so, need to calculate the amount of time to wait
+                                            final long elapsed = (System.currentTimeMillis() - starttime);
+                                            final long waitTime = maxWait - elapsed;
+                                            if (waitTime > 0)
+                                            {
+                                                latch.wait(waitTime);
+                                            }
                                         }
+                                    } else {
+                                        break;
                                     }
                                 }
                             } catch(InterruptedException e) {
@@ -1115,8 +1123,8 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
                             if(maxWait > 0 && ((System.currentTimeMillis() - starttime) >= maxWait)) {
                                 synchronized(this) {
                                     // Make sure allocate hasn't already assigned an object
-                                    // in a different thread
-                                    if (latch.getPair() == null) {
+                                    // in a different thread or permitted a new object to be created
+                                    if (latch.getPair() == null && !latch.mayCreate()) {
                                         // Remove latch from the allocation queue
                                         _allocationQueue.remove(latch);
                                     } else {
@@ -1173,8 +1181,10 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
                 }
                 synchronized (this) {
                     _numInternalProcessing--;
-                    latch.reset();
-                    _allocationQueue.add(0, latch);
+                    if (!newlyCreated) {
+                        latch.reset();
+                        _allocationQueue.add(0, latch);
+                    }
                     allocate();
                 }
                 if(newlyCreated) {
@@ -1245,7 +1255,18 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
     }
 
     /**
-     * Clears any objects sitting idle in the pool.
+     * Clears any objects sitting idle in the pool by removing them from the
+     * idle instance pool and then invoking the configured 
+     * {@link PoolableObjectFactory#destroyObject(Object)} method on each idle
+     * instance. 
+     * 
+     * <p> Implementation notes:
+     * <ul><li>This method does not destroy or effect in any way instances that are
+     * checked out of the pool when it is invoked.</li>
+     * <li>Invoking this method does not prevent objects being
+     * returned to the idle instance pool, even during its execution. It locks
+     * the pool only during instance removal. Additional instances may be returned
+     * while removed items are being destroyed.</li></ul></p>
      */
     public void clear() {
         List toDestroy = new ArrayList();
@@ -1627,6 +1648,12 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
         }
     }
 
+    /**
+     * Returns pool info including {@link #getNumActive()}, {@link #getNumIdle()}
+     * and a list of objects idle in the pool with their idle times.
+     * 
+     * @return string containing debug information
+     */
     synchronized String debugInfo() {
         StringBuffer buf = new StringBuffer();
         buf.append("Active: ").append(getNumActive()).append("\n");
@@ -1641,6 +1668,14 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
         return buf.toString();
     }
 
+    /** 
+     * Returns the number of tests to be performed in an Evictor run,
+     * based on the current value of <code>numTestsPerEvictionRun</code>
+     * and the number of idle instances in the pool.
+     * 
+     * @see #setNumTestsPerEvictionRun
+     * @return the number of tests for the Evictor to run
+     */
     private int getNumTests() {
         if(_numTestsPerEvictionRun >= 0) {
             return Math.min(_numTestsPerEvictionRun, _pool.size());
@@ -1656,6 +1691,10 @@ public class GenericObjectPool extends BaseObjectPool implements ObjectPool {
      * @see GenericObjectPool#setTimeBetweenEvictionRunsMillis
      */
     private class Evictor extends TimerTask {
+        /**
+         * Run pool maintenance.  Evict objects qualifying for eviction and then
+         * invoke {@link GenericObjectPool#ensureMinIdle()}.
+         */
         public void run() {
             try {
                 evict();
